#include <dirent.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/xattr.h>

static int do_stat(char *filename)
{
	char buf[4096];
	char contents[4096];
	int rc;
	struct stat statbuf;
	FILE *fp;
	ssize_t len;

	rc = stat(filename, &statbuf);
	if (rc)
		return rc;

	rc = lgetxattr(filename, "security.selinux", buf, sizeof(buf));
	if (rc < 0)
		return rc;

	fp = fopen(filename, "r");
	if (!fp)
		return -1;

	len = fread(contents, 1, sizeof(contents), fp);
	if (len < 0)
		return -1;

	contents[4095] = '\0';
	
	printf("ino=%lu context=%s contents=%s\n", (long unsigned int)statbuf.st_ino, buf, contents);

	return 0;
}

static int do_broken(char *filename)
{
	struct dirent *dirent;
	DIR *cwd;
	int rc;

	cwd = opendir("./");
	if (!cwd)
		return 1;
	dirent = readdir(cwd);
	dirent = dirent; //compiler warning that i wasn't using dirent

	rc = do_stat(filename);
	if (rc)
		return rc;

	closedir(cwd);

	return 0;
}

static int do_correct(char *filename)
{
	return do_stat(filename);
}

int main(int argc, char *argv[])
{
	int c;
	char *filename;
	int broken = 1;
	int correct = 0;

	while (1) {
		int option_index = 0;
		static struct option long_options[] = {
			{"broken",	no_argument,	0, 'b' },
			{"correct",	no_argument,	0, 'c' },
			{0,		0,		0, 0 }
		};

		c = getopt_long(argc, argv, "bc", long_options, &option_index);
		if (c == -1)
			break;
		switch (c) {
		case 'b':
			broken = 1;
			correct = 0;
			break;
		case 'c':
			broken = 0;
			correct = 1;
			break;
		default:
			printf("usage: %s [-b|-c] filename\n", argv[0]);
			return 0;
		};
	}
	if (optind != (argc-1)) {
		printf("must pass filename");
		return 1;
	}
	filename = argv[optind];

	if (broken)
		do_broken(filename);
	else if (correct)
		do_correct(filename);

	return 0;
}
